<?php
/**
 * Plugin Name:         RS Social Media
 * Description:         Create a post type for automatic re-sharing on social media.
 * Plugin URI:          https://plugins.seindal.dk/plugins/rs-social-media/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         rs-social-media
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             0.4.3
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

add_action( 'plugins_loaded', function() {
    class RS_Social_Media extends PluginBase {
        protected $plugin_file = __FILE__;


        /************************************************************************
         *
         *	Redirects for 'social-media' posts
         *
         *  Content of post should contain a HTML comment <!-- -->
         *  with either the URL to redirect to, or simply the numeric
         *  ID of a post to redirect to.
         *
         ************************************************************************/

        function do_template_redirect_action() {
            if ( ! is_single() )
                return;

            $post = get_post();

            if ( 'social-media' != $post->post_type )
                return;

            $target = NULL;
            if ( preg_match( '/<!--\s+(https?:\/\/\S+|\d+)\s+-->/', $post->post_content, $m ) )
                $target = $m[1];

            if ( $target && is_numeric( $target ) ) {
                $post = get_post( absint( $target ) );
                if ( $post && 'publish' == $post->post_status )
                    $target = get_permalink( $post );
            }

            if ( $target ) {
                wp_redirect( $target ?: site_url() );
                die;
            }
        }



        /*************************************************************
         *
         * Add 'Social Media' to post row
         *
         *************************************************************/

        function do_post_row_actions_filter_social_media( $actions, $post ) {
            if ( !is_post_type_viewable( $post->post_type ) )
                return $actions;

            if ( 'social-media' === $post->post_type )
                return $actions;

            $nonce = wp_create_nonce( 'social-media-create-nonce' );

            $url = $this->build_admin_url( 'admin-post.php', [
                'action' => 'social_media_create',
                'post' => $post->ID,
                '_wpnonce' => $nonce,
            ] );

            $link = sprintf( '<a href="%1$s">%2$s</a>',
                             esc_url( $url ),
                             esc_html( __( '→ SM', 'rs-social-media' ) )
            );
            $actions[ 'social_media_create' ] = $link;

            return $actions;
        }

        function do_page_row_actions_filter_duplicate_post( $actions, $post ) {
            return $this->do_post_row_actions_filter_social_media( $actions, $post );
        }


        /*************************************************************
         *
         * Setup the action handlers
         *
         *************************************************************/

        function do_admin_post_social_media_create_action() {
            if ( !( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( $_REQUEST['_wpnonce'], 'social-media-create-nonce' ) ) )
                wp_die( __( 'Not allowed', 'rs-social-media' ) );

            $post = NULL;

            if ( isset( $_GET['post'] ) && is_numeric( $_GET['post'] ) ) {
                $post_id = absint( $_GET['post'] );
                if ( $post_id )
                    $post = get_post( $post_id );

                if ( !is_post_type_viewable( $post->post_type ) )
                    $post = NULL;

            }

            if ( empty( $post ) )
                wp_die( __( 'Error', 'rs-social-media' ) );

            // Basic changes
            $data = [];

            $data['ID'] = 0;
            $data['post_type'] = 'social-media';
            $data['post_status'] = 'draft';
            $data['post_title'] = $post->post_title;
            $data['post_excerpt'] = $post->post_excerpt;
            $data['post_content'] = sprintf( "<!-- wp:paragraph -->
<p>%s</p>

<!-- /wp:paragraph --><!-- wp:html -->
<!-- %s -->
<!-- /wp:html -->
", esc_html( $post->post_excerpt), get_permalink( $post ) );

            // Find taxonomies for social-media
            $taxonomies = array_map(
                fn($t) => $t->name,
                array_filter(
                    get_taxonomies( [ 'show_ui' => true ], 'objects' ),
                    fn($t) => in_array( 'social-media', $t->object_type ) && in_array( $post->post_type, $t->object_type )
                )
            );

            foreach ( $taxonomies as $taxonomy ) {
                $terms = wp_get_post_terms( $post_id, $taxonomy, [ 'fields' => 'ids' ] );
                if ( ! empty( $terms ) ) {
                    $data['tax_input'][$taxonomy] = $terms;
                }
            }

            $thumbnail = $post->_thumbnail_id;
            if ( $thumbnail )
                $data['meta_input']['_thumbnail_id'] = $thumbnail;

            $id = wp_insert_post( $data, true );
            if( is_wp_error( $id ) ) {
                echo $id->get_error_message();
                return;
            }

            $url = $this->build_admin_url( 'post.php', [ 'post' => $id, 'action' => 'edit' ] );
            wp_redirect( $url );
            exit;
        }


        /*************************************************************
         *
         * Registers the `social-media` post type.
         *
         *************************************************************/

        function do_wp_loaded_action_social_media_taxonomies() {
            register_taxonomy_for_object_type( 'post_tag', 'social-media' );
        }

        function do_init_action_social_media() {
            $supports = [
                'title',
                'editor',
                'comments',
                'revisions',
                'author',
                'excerpt',
                'page-attributes',
                'thumbnail',
                'custom-fields',
            ];

            register_post_type(
                'social-media',
                [
                    'labels'                => [
                        'name'                  => __( 'Social Media', 'rs-social-media' ),
                        'singular_name'         => __( 'Social Media', 'rs-social-media' ),
                        'all_items'             => __( 'All Social Media', 'rs-social-media' ),
                        'archives'              => __( 'Social Media Archives', 'rs-social-media' ),
                        'attributes'            => __( 'Social Media Attributes', 'rs-social-media' ),
                        'insert_into_item'      => __( 'Insert into Social Media', 'rs-social-media' ),
                        'uploaded_to_this_item' => __( 'Uploaded to this Social Media', 'rs-social-media' ),
                        'featured_image'        => _x( 'Featured Image', 'rs-social-media', 'rs-social-media' ),
                        'set_featured_image'    => _x( 'Set featured image', 'rs-social-media', 'rs-social-media' ),
                        'remove_featured_image' => _x( 'Remove featured image', 'rs-social-media', 'rs-social-media' ),
                        'use_featured_image'    => _x( 'Use as featured image', 'rs-social-media', 'rs-social-media' ),
                        'filter_items_list'     => __( 'Filter Social Media list', 'rs-social-media' ),
                        'items_list_navigation' => __( 'Social Media list navigation', 'rs-social-media' ),
                        'items_list'            => __( 'Social Media list', 'rs-social-media' ),
                        'new_item'              => __( 'New Social Media', 'rs-social-media' ),
                        'add_new'               => __( 'Add New', 'rs-social-media' ),
                        'add_new_item'          => __( 'Add New Social Media', 'rs-social-media' ),
                        'edit_item'             => __( 'Edit Social Media', 'rs-social-media' ),
                        'view_item'             => __( 'View Social Media', 'rs-social-media' ),
                        'view_items'            => __( 'View Social Media', 'rs-social-media' ),
                        'search_items'          => __( 'Search Social Media', 'rs-social-media' ),
                        'not_found'             => __( 'No Social Media found', 'rs-social-media' ),
                        'not_found_in_trash'    => __( 'No Social Media found in trash', 'rs-social-media' ),
                        'parent_item_colon'     => __( 'Parent Social Media:', 'rs-social-media' ),
                        'menu_name'             => __( 'Social Media', 'rs-social-media' ),
                    ],
                    'public'                => true,
                    'hierarchical'          => false,
                    'show_ui'               => true,
                    'show_in_nav_menus'     => true,
                    'supports'              => $supports,
                    'has_archive'           => true,
                    'rewrite'               => true,
                    'query_var'             => true,
                    'menu_position'         => null,
                    'menu_icon'             => 'dashicons-share-alt',
                    'show_in_rest'          => true,
                    'rest_base'             => 'social-media',
                    'rest_controller_class' => 'WP_REST_Posts_Controller',
                ]
            );

        }

        /**
         * Sets the post updated messages for the `social_media` post type.
         *
         * @param  array $messages Post updated messages.
         * @return array Messages for the `rs_social_media` post type.
         */

        function do_post_updated_messages_filter_social_media( $messages ) {
            global $post;

            $permalink = get_permalink( $post );

            $messages['social-media'] = [
                0  => '', // Unused. Messages start at index 1.
                /* translators: %s: post permalink */
                1  => sprintf( __( 'Social Media updated. <a target="_blank" href="%s">View Social Media</a>', 'rs-social-media' ), esc_url( $permalink ) ),
                2  => __( 'Custom field updated.', 'rs-social-media' ),
                3  => __( 'Custom field deleted.', 'rs-social-media' ),
                4  => __( 'Social Media updated.', 'rs-social-media' ),
                /* translators: %s: date and time of the revision */
                5  => isset( $_GET['revision'] ) ? sprintf( __( 'Social Media restored to revision from %s', 'rs-social-media' ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false, // phpcs:ignore WordPress.Security.NonceVerification.Recommended
                /* translators: %s: post permalink */
                6  => sprintf( __( 'Social Media published. <a href="%s">View Social Media</a>', 'rs-social-media' ), esc_url( $permalink ) ),
                7  => __( 'Social Media saved.', 'rs-social-media' ),
                /* translators: %s: post permalink */
                8  => sprintf( __( 'Social Media submitted. <a target="_blank" href="%s">Preview Social Media</a>', 'rs-social-media' ), esc_url( add_query_arg( 'preview', 'true', $permalink ) ) ),
                /* translators: 1: Publish box date format, see https://secure.php.net/date 2: Post permalink */
                9  => sprintf( __( 'Social Media scheduled for: <strong>%1$s</strong>. <a target="_blank" href="%2$s">Preview Social Media</a>', 'rs-social-media' ), date_i18n( __( 'M j, Y @ G:i', 'rs-social-media' ), strtotime( $post->post_date ) ), esc_url( $permalink ) ),
                /* translators: %s: post permalink */
                10 => sprintf( __( 'Social Media draft updated. <a target="_blank" href="%s">Preview Social Media</a>', 'rs-social-media' ), esc_url( add_query_arg( 'preview', 'true', $permalink ) ) ),
            ];

            return $messages;
        }


        /**
         * Sets the bulk post updated messages for the `social_media` post type.
         *
         * @param  array $bulk_messages Arrays of messages, each keyed by the corresponding post type. Messages are
         *                              keyed with 'updated', 'locked', 'deleted', 'trashed', and 'untrashed'.
         * @param  int[] $bulk_counts   Array of item counts for each message, used to build internationalized strings.
         * @return array Bulk messages for the `rs_social_media` post type.
         */

        protected $bulk_post_updated_messages_callback_arguments = 2;

        function do_bulk_post_updated_messages_filter_social_media( $bulk_messages, $bulk_counts ) {
            global $post;

            $bulk_messages['social-media'] = [
                /* translators: %s: Number of Social Media. */
                'updated'   => _n( '%s Social Media updated.', '%s Social Media updated.', $bulk_counts['updated'], 'rs-social-media' ),
                'locked'    => ( 1 === $bulk_counts['locked'] ) ? __( '1 Social Media not updated, somebody is editing it.', 'rs-social-media' ) :
                /* translators: %s: Number of Social Media. */
                _n( '%s Social Media not updated, somebody is editing it.', '%s Social Media not updated, somebody is editing them.', $bulk_counts['locked'], 'rs-social-media' ),
                /* translators: %s: Number of Social Media. */
                'deleted'   => _n( '%s Social Media permanently deleted.', '%s Social Media permanently deleted.', $bulk_counts['deleted'], 'rs-social-media' ),
                /* translators: %s: Number of Social Media. */
                'trashed'   => _n( '%s Social Media moved to the Trash.', '%s Social Media moved to the Trash.', $bulk_counts['trashed'], 'rs-social-media' ),
                /* translators: %s: Number of Social Media. */
                'untrashed' => _n( '%s Social Media restored from the Trash.', '%s Social Media restored from the Trash.', $bulk_counts['untrashed'], 'rs-social-media' ),
            ];

            return $bulk_messages;
        }



    }

    new RS_Social_Media();
} );
